#ifndef _update_entity_h
#define _update_entity_h

#include "typeclass_helpers.h"
#include "vmath.h"

typedef struct {
    void (*const spawn)(void* self, Vector at);
    void (*const update)(void* self, float dt);
    void (*const start)(void* self);
} IEntityBehaviour;

typedef struct {
    void* data;
    IEntityBehaviour const* tc;
} BehaviourEntity;

#define impl_BehaviourEntity_for(T, start_f, update_f, spawn_f)\
static inline BehaviourEntity T##_as_BehaviourEntity(T* x) {\
    TC_FN_TYPECHECK(void, start_f, T*);\
    TC_FN_TYPECHECK(void, update_f, T*, float);\
    static IEntityBehaviour const tc = {\
        .spawn =   (void(*const)(void*, Vector)) spawn_f,\
        .update =  (void(*const)(void*, float))  update_f,\
        .start =   (void(*const)(void*))         start_f\
    };\
    return (BehaviourEntity){.tc = &tc, .data = x};\
}\

#endif // !_update_entity_h
